/*
 *  Permission hereby granted to use and change this software, and make
 *  derivative works based on this code, as long as it mentions initial
 *  author - Roman Mitnitski (mitnits@shani.net) 
 * 
 *  This file uses code borrowed from Rob Nation's xpmroot utility
 *  Thank, Rob. Here's his copyright notice:
 *
 *   Copyright 1993, Rob Nation
 *   You may use this file for anything you want, as long as the copyright
 *   is kept intact. No guarantees of any sort are made in any way regarding
 *   this program or anything related to it.
 *
 * 
 *  ROMAN MITNITSKI AND ROB NATION MAKE NO WARRANTY OF ANY KIND 
 *  WITH REGARD TO THIS SOFWARE, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE.  Roman Mitnitski and Rob Nation will not be liable 
 *  for errors contained herein or direct, indirect, special, incidental 
 *  or consequential damages in connection with the furnishing,
 *  performance, or use of this program.               
 */

#include "forms.h"
#include "xbackdrop.h"
#include "newbg.h"
#include "dummy.xpm"
#include <X11/xpm.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

typedef struct backdrop_
{
  int deflt;
  char * name;
  char * xpmfile;
} backdrop;
extern FD_newbg * fd_newbg;
extern backdrop backdrops[256];
extern int backd_cnt;
extern FL_OBJECT * pixmap, *pixframe, *pixlabel, * namelist;
extern FL_FORM * mainform;
extern Window Mainwin;
extern FD_xbackdrop * fd_xbackdrop;
static XpmAttributes attributes;

int save_colors = 0;
Display *dpy;
int screen;
Window root;
char *display_name = NULL;
void SetRootWindow(char *tline);
Pixmap rootXpm;

void SetRootWindow(char *tline)
{
  XWindowAttributes root_attr;
  XpmAttributes xpm_attributes;
  Pixmap shapeMask;
  int val;

  XGetWindowAttributes(dpy,root,&root_attr);
  xpm_attributes.colormap = root_attr.colormap;
  xpm_attributes.valuemask = XpmSize | XpmReturnPixels|XpmColormap;
  if((val = XpmReadFileToPixmap(dpy,root, tline,
			 &rootXpm, &shapeMask, 
			 &xpm_attributes))!= XpmSuccess) 
    {
      if(val == XpmOpenFailed)
	fprintf(stderr, "Couldn't open pixmap file\n");
      else if(val == XpmColorFailed)
	fprintf(stderr, "Couldn't allocate required colors\n");
      else if(val == XpmFileInvalid)
	fprintf(stderr, "Invalid Format for an Xpm File\n");
      else if(val == XpmColorError)
	fprintf(stderr, "Invalid Color specified in Xpm FIle\n");
      else if(val == XpmNoMemory)
	fprintf(stderr, "Insufficient Memory\n");
      exit(1);
    }

  XSetWindowBackgroundPixmap(dpy, root, rootXpm);
  save_colors = 1;
  XClearWindow(dpy,root);

}


#ifdef NEEDS_STRCASECMP
int strcasecmp(char *s1,char *s2)
{
  int c1,c2;
  int n,n2;

  n=strlen(s1);
  n2=strlen(s2);
  if(n!=n2)
    return 1;

  for (;;)
    {
      c1 = *s1; 
      c2 = *s2;
      if (!c1 || !c2) 
	return(c1 - c2);
      if (isupper(c1)) 
	c1 = 'a' - 1 + (c1 & 31);
      if (isupper(c2)) 
	c2 = 'a' - 1 + (c2 & 31);
      if (c1 != c2) 
	return(c1 - c2);
      n--,s1++,s2++;
    }
}
#endif

/* callbacks for form xbackdrop */
void selected_pixmap(FL_OBJECT *ob, long data)
{
  int linenum;
  Display * dpy;
  int Error;

  Pixmap mask;
  Pixmap icon;
  Colormap cmap;
  dpy=fl_get_display();
  cmap=DefaultColormap(dpy,DefaultScreen(dpy));
  linenum=fl_get_browser(ob);
#if FL_REVISION>=80
  fl_set_pixmap_data(pixmap  ,dummy);
#else
  fl_set_pixmap_data(pixmap,  241, 217  ,dummy);
#endif
  fl_redraw_object(pixlabel);
  fl_redraw_object(pixframe);

  attributes.valuemask=XpmReturnPixels;
  if (attributes.npixels>0)
    { /* Free XPM colors */
      XFreeColors(dpy, cmap, attributes.pixels, 
		   attributes.npixels, 0L);     
    }
  Error=XpmReadFileToPixmap(dpy, Mainwin,backdrops[linenum-1].xpmfile,
                               &icon, &mask, &attributes);


  if (Error==0)
    {
      XFreePixmap(dpy,icon);
      if (mask!=None) XFreePixmap(dpy,mask);
    }

  fl_set_pixmap_file(pixmap,backdrops[linenum-1].xpmfile);
  /*  if (attributes.npixels>0)
    {  Free XPM colors 
      XFreeColors(dpy, cmap, attributes.pixels, 
		   attributes.npixels, 0L);     
    }
*/
}

void add_item(char * newname,char *newfile)
{
  backdrops[backd_cnt].name=(char *)calloc(1,strlen(newname)+1);
  backdrops[backd_cnt].xpmfile=(char *)calloc(1,strlen(newfile)+1);
  strcpy(backdrops[backd_cnt].name, newname);
  strcpy(backdrops[backd_cnt].xpmfile, newfile);
  backd_cnt++;
  fl_add_browser_line(fd_xbackdrop->backlist,newname);
}

void apply(FL_OBJECT *ob, long pdata)
{
  int linenum;
  Atom prop, type;
  int format,i;
  unsigned long length, after;
  unsigned char *data;
 
  dpy=fl_get_display();
  for (i=0;i<256;i++) backdrops[i].deflt=0;
  linenum=fl_get_browser(fd_xbackdrop->backlist)-1;
  if (linenum>=0) 
    {
      screen = DefaultScreen(dpy);
      root = RootWindow(dpy, screen);
      
      SetRootWindow(backdrops[linenum].xpmfile);
      
      prop = XInternAtom(dpy, "_XSETROOT_ID", False);
      
      (void)XGetWindowProperty(dpy, root, prop, 0L, 1L, True, AnyPropertyType,
			       &type, &format, &length, &after, &data);
      if ((type == XA_PIXMAP) && (format == 32) && (length == 1) && (after == 0))
	XKillClient(dpy, *((Pixmap *)data));
      
      XChangeProperty(dpy, root, prop, XA_PIXMAP, 32, PropModeReplace,
		      (unsigned char *) &rootXpm, 1);
      XDestroyWindow(dpy,Mainwin);
      XSetCloseDownMode(dpy, RetainPermanent);
      backdrops[linenum].deflt=1;
    }
  update_list();
  exit(0);  
}

void cancel(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
  exit(0);
}

void rm(FL_OBJECT *ob, long data)
{
  int linenum;
  Display * dpy;
  int i;
  Pixmap mask;
  Pixmap icon;
  Colormap cmap;

  dpy=fl_get_display();
  cmap=DefaultColormap(dpy,DefaultScreen(dpy));
  linenum=fl_get_browser(fd_xbackdrop->backlist);
  if (linenum==0) 
    {
      fl_show_alert("Please select the item to remove","",
		    "",1); 
      return;
    }
#if FL_REVISION>=80
  fl_set_pixmap_data(pixmap  ,dummy);
#else
  fl_set_pixmap_data(pixmap,  241, 217  ,dummy);
#endif
  fl_redraw_object(pixlabel);
  fl_redraw_object(pixframe);

  attributes.valuemask=XpmReturnPixels;
  if (attributes.npixels>0)
    { /* Free XPM colors */
      XFreeColors(dpy, cmap, attributes.pixels, 
		   attributes.npixels, 0L);     
    }
  attributes.npixels=0;
  fl_delete_browser_line(fd_xbackdrop->backlist,linenum);
  free(backdrops[linenum-1].name);
  free(backdrops[linenum-1].xpmfile);
  for (i=linenum-1;i<backd_cnt-1;i++)
    {
      backdrops[i]=backdrops[i+1];
    }
}

void new(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
  fl_deactivate_form(fl_win_to_form(Mainwin));
  fl_show_form(fd_newbg->newbg,FL_PLACE_CENTER,FL_TRANSIENT,"Select New Background");
}



